@@ -8,7 +8,7 @@ import textwrap
 import threading
 from logging import Logger
 from pathlib import Path
-from typing import TYPE_CHECKING, Any, ClassVar, Iterable, Mapping, cast
+from typing import TYPE_CHECKING, Any, ClassVar, Iterable, cast
 
 from pyproject_hooks import BuildBackendHookCaller
 
@@ -19,7 +19,6 @@ from pdm.models.in_process import get_sys_config_paths
 from pdm.models.requirements import Requirement, parse_requirement
 from pdm.models.working_set import WorkingSet
 from pdm.termui import logger
-from pdm.utils import create_tracked_tempdir
 
 if TYPE_CHECKING:
     from pdm.environments import BaseEnvironment
@@ -73,7 +72,7 @@ def build_error(e: subprocess.CalledProcessError) -> BuildError:
     """
     output = cast("list[str]", e.output)
     errors: list[str] = []
-    if output[-1].strip().startswith("ModuleNotFoundError"):
+    if output and output[-1].strip().startswith("ModuleNotFoundError"):
         package = output[-1].strip().split()[-1]
         errors.append(
             f"Module {package} is missing, please make sure it is specified in the "
@@ -162,20 +161,18 @@ class EnvBuilder:
         _requires: list[str]
         _prefix: _Prefix
 
-    @classmethod
-    def get_shared_env(cls, key: int) -> str:
-        if key in cls._shared_envs:
-            logger.debug("Reusing shared build env: %s", cls._shared_envs[key])
-            return cls._shared_envs[key]
+    def get_shared_env(self, key: int) -> str:
+        if key in self._shared_envs:
+            logger.debug("Reusing shared build env: %s", self._shared_envs[key])
+            return self._shared_envs[key]
         # We don't save the cache here, instead it will be done after the installation
         # finished.
-        return create_tracked_tempdir("-shared", "pdm-build-env-")
+        return self._env.project.core.create_temp_dir("-shared", "pdm-build-env-")
 
-    @classmethod
-    def get_overlay_env(cls, key: str) -> str:
-        if key not in cls._overlay_envs:
-            cls._overlay_envs[key] = create_tracked_tempdir("-overlay", "pdm-build-env-")
-        return cls._overlay_envs[key]
+    def get_overlay_env(self, key: str) -> str:
+        if key not in self._overlay_envs:
+            self._overlay_envs[key] = self._env.project.core.create_temp_dir("-overlay", "pdm-build-env-")
+        return self._overlay_envs[key]
 
     def __init__(self, src_dir: str | Path, environment: BaseEnvironment) -> None:
         """If isolated is True(default), the builder will set up a *clean* environment.
@@ -185,6 +182,7 @@ class EnvBuilder:
         self.executable = self._env.interpreter.executable.as_posix()
         self.src_dir = src_dir
         self.isolated = environment.project.config["build_isolation"]
+        self.config_settings = environment.project.core.state.config_settings
         mode = "Isolated" if self.isolated else "Non-isolated"
         logger.info("Preparing environment(%s mode) for PEP 517 build...", mode)
         try:
@@ -304,19 +302,14 @@ class EnvBuilder:
             if key not in self._shared_envs:
                 self._shared_envs[key] = path
 
-    def prepare_metadata(self, out_dir: str, config_settings: Mapping[str, Any] | None = None) -> str:
+    def prepare_metadata(self, out_dir: str) -> str:
         """Prepare metadata and store in the out_dir.
         Some backends doesn't provide that API, in that case the metadata will be
         retrieved from the built result.
         """
         raise NotImplementedError("Should be implemented in subclass")
 
-    def build(
-        self,
-        out_dir: str,
-        config_settings: Mapping[str, Any] | None = None,
-        metadata_directory: str | None = None,
-    ) -> str:
+    def build(self, out_dir: str, metadata_directory: str | None = None) -> str:
         """Build and store the artifact in out_dir,
         return the absolute path of the built result.
         """
