@@ -4,7 +4,6 @@ import argparse
 import csv
 import io
 import json
-import re
 from collections import defaultdict
 from fnmatch import fnmatch
 from typing import Iterable, Mapping, Sequence
@@ -18,6 +17,7 @@ from pdm.cli.utils import (
     build_dependency_graph,
     check_project_file,
     get_dist_location,
+    normalize_pattern,
     show_dependency_graph,
 )
 from pdm.compat import importlib_metadata as im
@@ -29,10 +29,6 @@ from pdm.project import Project
 SUBDEP_GROUP_LABEL = ":sub"
 
 
-def normalize_pattern(pattern: str) -> str:
-    return re.sub(r"[^A-Za-z0-9*?]+", "-", pattern).lower()
-
-
 class Command(BaseCommand):
     """List packages installed in the current working set"""
 
@@ -160,7 +156,9 @@ class Command(BaseCommand):
             r for g in selected_groups if g != SUBDEP_GROUP_LABEL for r in project.get_dependencies(g).values()
         ]
         if options.resolve:
-            candidates = actions.resolve_candidates_from_lockfile(project, requirements)
+            candidates = actions.resolve_candidates_from_lockfile(
+                project, requirements, groups=selected_groups - {SUBDEP_GROUP_LABEL}
+            )
             packages: Mapping[str, im.Distribution] = {
                 k: c.prepare(project.environment).metadata for k, c in candidates.items()
             }
@@ -253,11 +251,10 @@ class Command(BaseCommand):
         ui = project.core.ui
 
         # Order based on a field key.
-        if options.sort:
-            keys = parse_comma_separated_string(options.sort)
-            if not all(key in Listable.KEYS for key in keys):
-                raise PdmUsageError(f"--sort key must be one of: {','.join(Listable.KEYS)}")
-            records.sort(key=lambda d: tuple(d[key].casefold() for key in keys))
+        keys = parse_comma_separated_string(options.sort) if options.sort else ["name"]
+        if not all(key in Listable.KEYS for key in keys):
+            raise PdmUsageError(f"--sort key must be one of: {','.join(Listable.KEYS)}")
+        records.sort(key=lambda d: tuple(d[key].casefold() for key in keys))
 
         # Write CSV
         if options.csv:
@@ -281,11 +278,9 @@ class Command(BaseCommand):
             try:
                 ui.echo(text_body, highlight=True)
             except UnicodeEncodeError:
-                ui.echo(
+                ui.error(
                     "Markdown output contains non-ASCII characters. "
                     "Setting env var PYTHONIOENCODING to 'utf8' may fix this.",
-                    err=True,
-                    style="error",
                 )
                 ui.echo(text_body.encode().decode("ascii", errors="ignore"), highlight=True)
                 ui.echo("**Problem decoding file as UTF-8.  Some characters may be omit.**")
@@ -341,7 +336,15 @@ class Listable:
         # so generate a pipe separated list (to avoid complexity with CSV export).
         self.licenses: str | None = dist.metadata["License"]
         self.licenses = None if self.licenses == "UNKNOWN" else self.licenses
-        if not self.licenses:
+
+        # Sometimes package metadata contains the full license text.
+        # e.g. license = { file="LICENSE" } in pyproject.toml
+        # To identify this, check for newlines or very long strings.
+        # 50 chars is picked because the longest OSI license (WTFPL) full name is 43 characters.
+        is_full_text = self.licenses and "\n" in self.licenses or len(self.licenses or "") > 50
+
+        # If that is the case, look at the classifiers instead.
+        if not self.licenses or is_full_text:
             classifier_licenses = [v for v in dist.metadata.get_all("Classifier", []) if v.startswith("License")]
             alternatives = [parts.split("::") for parts in classifier_licenses]
             alternatives = [part[-1].strip() for part in alternatives if part]
