@@ -255,7 +255,7 @@ def build_dependency_graph(
 
 
 def specifier_from_requirement(requirement: Requirement) -> str:
-    return str(requirement.specifier or "Any")
+    return str(requirement.specifier) or "Any"
 
 
 def add_package_to_tree(
@@ -676,7 +676,7 @@ def format_resolution_impossible(err: ResolutionImpossible) -> str:
             pyspec &= req.specifier
             info_lines.add(f"  {req.as_line()} (from {parent!r})")
         result.extend(sorted(info_lines))
-        if pyspec.is_impossible:
+        if pyspec.is_empty():
             result.append("Consider changing the version specifiers of the dependencies to be compatible")
         else:
             result.append(
@@ -702,7 +702,7 @@ def format_resolution_impossible(err: ResolutionImpossible) -> str:
     result.extend(sorted(info_lines))
     result.append(
         "To fix this, you could loosen the dependency version constraints in "
-        "pyproject.toml. See https://pdm-project.org/latest/usage/dependency/"
+        "pyproject.toml. See https://pdm-project.org/en/latest/usage/lockfile/"
         "#solve-the-locking-failure for more details."
     )
     return "\n".join(result)
@@ -792,3 +792,8 @@ def populate_requirement_names(req_mapping: dict[str, Requirement]) -> None:
         if key and key.startswith(":empty:"):
             req_mapping[req.identify()] = req
             del req_mapping[key]
+
+
+def normalize_pattern(pattern: str) -> str:
+    """Normalize a pattern to a valid name for a package."""
+    return re.sub(r"[^A-Za-z0-9*?]+", "-", pattern).lower()
