@@ -5,22 +5,27 @@ r"""
  / ____/ /_/ / /  / /
 /_/   /_____/_/  /_/
 """
+
 from __future__ import annotations
 
 import argparse
+import contextlib
+import dataclasses as dc
 import importlib
 import itertools
 import os
 import pkgutil
 import sys
+from datetime import datetime
 from pathlib import Path
-from typing import TYPE_CHECKING, cast
+from tempfile import TemporaryDirectory
+from typing import TYPE_CHECKING, Iterator, cast
 
 from resolvelib import Resolver
 
 from pdm import termui
 from pdm.__version__ import __version__
-from pdm.cli.options import ignore_python_option, pep582_option, verbose_option
+from pdm.cli.options import ignore_python_option, no_cache_option, pep582_option, verbose_option
 from pdm.cli.utils import ArgumentParser, ErrorArgumentParser
 from pdm.compat import importlib_metadata
 from pdm.exceptions import PdmArgumentError, PdmUsageError
@@ -39,6 +44,16 @@ if TYPE_CHECKING:
 COMMANDS_MODULE_PATH = importlib.import_module("pdm.cli.commands").__path__
 
 
+@dc.dataclass
+class State:
+    """State of the core object."""
+
+    config_settings: dict[str, Any] | None = None
+    """The config settings map shared by all packages"""
+    exclude_newer: datetime | None = None
+    """The exclude newer than datetime for the lockfile"""
+
+
 class Core:
     """A high level object that manages all classes and configurations"""
 
@@ -53,10 +68,39 @@ class Core:
 
     def __init__(self) -> None:
         self.version = __version__
-        self.ui = termui.UI()
+        self.exit_stack = contextlib.ExitStack()
+        self.ui = termui.UI(exit_stack=self.exit_stack)
+        self._states = [State()]
+        self.exit_stack.callback(setattr, self, "config_settings", None)
         self.init_parser()
         self.load_plugins()
 
+    @property
+    def state(self) -> State:
+        """Get the current state object."""
+        return self._states[-1]
+
+    @contextlib.contextmanager
+    def push_state(self, __empty: bool = False, /, **kwargs: Any) -> Iterator[State]:  # pragma: no cover
+        """Push a new state object to the stack.
+
+        Args:
+            __empty (bool): Whether to make an empty state.
+            **kwargs: The new attributes to set to the state object.
+        """
+        if __empty:
+            new_state = State(**kwargs)
+        else:
+            new_state = dc.replace(self.state, **kwargs)
+        self._states.append(new_state)
+        try:
+            yield new_state
+        finally:
+            self._states.pop()
+
+    def create_temp_dir(self, *args: Any, **kwargs: Any) -> str:
+        return self.exit_stack.enter_context(TemporaryDirectory(*args, **kwargs))
+
     def init_parser(self) -> None:
         self.parser = ErrorArgumentParser(
             prog="pdm",
@@ -79,6 +123,7 @@ class Core:
             help="Specify another config file path [env var: PDM_CONFIG_FILE] ",
         )
         verbose_option.add_to_parser(self.parser)
+        no_cache_option.add_to_parser(self.parser)
         ignore_python_option.add_to_parser(self.parser)
         pep582_option.add_to_parser(self.parser)
 
@@ -134,8 +179,15 @@ class Core:
 
         self.ui.set_verbosity(options.verbose)
         self.ui.set_theme(project.global_config.load_theme())
+        self.ui.log_dir = os.path.expanduser(cast(str, project.config["log_dir"]))
+        os.makedirs(self.ui.log_dir, exist_ok=True)
 
         command = cast("BaseCommand | None", getattr(options, "command", None))
+        self.state.config_settings = getattr(options, "config_setting", None)
+
+        if options.no_cache:
+            project.cache_dir = Path(self.create_temp_dir(prefix="pdm-cache-"))
+
         hooks = HookManager(project, getattr(options, "skip", None))
         hooks.try_emit("pre_invoke", command=command.name if command else None, options=options)
 
@@ -175,7 +227,6 @@ class Core:
         **extra: Any,
     ) -> None:
         """The main entry function"""
-
         if args is None:
             args = []
         args = self._get_cli_args(args, obj)
@@ -211,12 +262,7 @@ class Core:
                 err=True,
             )
             if should_show_tb:
-                self.ui.echo(
-                    "Add '-v' to see the detailed traceback",
-                    style="warning",
-                    err=True,
-                    verbosity=termui.Verbosity.NORMAL,
-                )
+                self.ui.warn("Add '-v' to see the detailed traceback", verbosity=termui.Verbosity.NORMAL)
             sys.exit(1)
         else:
             if project.config["check_update"] and not is_in_zipapp():
@@ -283,13 +329,13 @@ class Core:
             try:
                 plugin.load()(self)
             except Exception as e:
-                self.ui.echo(
+                self.ui.error(
                     f"Failed to load plugin {plugin.name}={plugin.value}: {e}",
-                    style="error",
-                    err=True,
                 )
 
 
 def main(args: list[str] | None = None) -> None:
     """The CLI entry function"""
-    return Core().main(args or sys.argv[1:])
+    core = Core()
+    with core.exit_stack:
+        return core.main(args or sys.argv[1:])
