@@ -1,6 +1,7 @@
 """
 Utility functions
 """
+
 from __future__ import annotations
 
 import atexit
@@ -16,13 +17,15 @@ import sysconfig
 import tempfile
 import urllib.parse as parse
 import warnings
+from datetime import datetime, timezone
 from os import name as os_name
 from pathlib import Path
-from typing import TYPE_CHECKING
+from typing import TYPE_CHECKING, Mapping
 
 from packaging.version import Version, _cmpkey
 
 from pdm.compat import importlib_metadata
+from pdm.exceptions import PDMDeprecationWarning, PdmException
 
 if TYPE_CHECKING:
     from re import Match
@@ -38,7 +41,13 @@ try:
 except Exception:
     from packaging import __version__ as _packaging_version
 
-PACKAGING_22 = Version(_packaging_version) >= Version("22")
+
+@functools.lru_cache(maxsize=1024)
+def parse_version(version: str) -> Version:
+    return Version(version)
+
+
+PACKAGING_22 = parse_version(_packaging_version) >= parse_version("22")
 
 
 def create_tracked_tempdir(suffix: str | None = None, prefix: str | None = None, dir: str | None = None) -> str:
@@ -220,17 +229,19 @@ def path_to_url(path: str) -> str:
     return url
 
 
-def expand_env_vars(credential: str, quote: bool = False) -> str:
+def expand_env_vars(credential: str, quote: bool = False, env: Mapping[str, str] | None = None) -> str:
     """A safe implementation of env var substitution.
     It only supports the following forms:
 
         ${ENV_VAR}
 
-    Neither $ENV_VAR and %ENV_VAR is not supported.
+    Neither $ENV_VAR and %ENV_VAR is supported.
     """
+    if env is None:
+        env = os.environ
 
     def replace_func(match: Match) -> str:
-        rv = os.getenv(match.group(1), match.group(0))
+        rv = env.get(match.group(1), match.group(0))
         return parse.quote(rv) if quote else rv
 
     return re.sub(r"\$\{(.+?)\}", replace_func, credential)
@@ -246,7 +257,7 @@ def expand_env_vars_in_auth(url: str) -> str:
     return parse.urlunparse((scheme, netloc, path, params, query, fragment))
 
 
-@functools.lru_cache()
+@functools.lru_cache
 def path_replace(pattern: str, replace_with: str, dest: str) -> str:
     """Safely replace the pattern in a path with given string.
 
@@ -336,7 +347,7 @@ def normalize_name(name: str, lowercase: bool = True) -> str:
 
 def comparable_version(version: str) -> Version:
     """Normalize a version to make it valid in a specifier."""
-    parsed = Version(version)
+    parsed = parse_version(version)
     if parsed.local is not None:
         # strip the local part
         parsed._version = parsed._version._replace(local=None)
@@ -393,16 +404,17 @@ def is_url(url: str) -> bool:
     return bool(parse.urlparse(url).scheme)
 
 
-@functools.lru_cache()
-def fs_supports_symlink() -> bool:
-    if not hasattr(os, "symlink"):
+@functools.lru_cache
+def fs_supports_link_method(method: str) -> bool:
+    if not hasattr(os, method):
         return False
     if sys.platform == "win32":
-        with tempfile.NamedTemporaryFile(prefix="TmP") as tmp_file:
-            temp_dir = os.path.dirname(tmp_file.name)
-            dest = os.path.join(temp_dir, "{}-{}".format(tmp_file.name, "b"))
+        with tempfile.TemporaryDirectory(prefix="TmP") as temp_dir:
+            with open(src := os.path.join(temp_dir, "a"), "w") as tmp_file:
+                tmp_file.write("foo")
+            dest = f"{src}-link"
             try:
-                os.symlink(tmp_file.name, dest)
+                getattr(os, method)(src, dest)
                 return True
             except (OSError, NotImplementedError):
                 return False
@@ -417,9 +429,9 @@ def deprecation_warning(message: str, stacklevel: int = 1, raise_since: str | No
     from pdm.__version__ import __version__
 
     if raise_since is not None:
-        if Version(__version__) >= Version(raise_since):
-            raise FutureWarning(message)
-    warnings.warn(message, FutureWarning, stacklevel=stacklevel + 1)
+        if parse_version(__version__) >= parse_version(raise_since):
+            raise PDMDeprecationWarning(message)
+    warnings.warn(message, PDMDeprecationWarning, stacklevel=stacklevel + 1)
 
 
 def is_pip_compatible_with_python(python_version: Version | str) -> bool:
@@ -464,8 +476,12 @@ def validate_project_name(name: str) -> bool:
 
 def sanitize_project_name(name: str) -> str:
     """Sanitize the project name and remove all illegal characters"""
-    pattern = r"[^a-zA-Z0-9\-_\.]"
-    return re.sub(pattern, "", name)
+    pattern = r"[^a-zA-Z0-9\-_\.]+"
+    result = re.sub(pattern, "-", name)
+    result = re.sub(r"^[\._-]|[\._-]$", "", result)
+    if not result:
+        raise PdmException(f"Invalid project name: {name}")
+    return result
 
 
 def is_conda_base() -> bool:
@@ -481,3 +497,42 @@ def is_conda_base_python(python: Path) -> bool:
     except ValueError:
         return False
     return True
+
+
+def filtered_sources(sources: list[RepositoryConfig], package: str | None) -> list[RepositoryConfig]:
+    """Get matching sources based on the index attribute."""
+    source_preferences = [(s, _source_preference(package, s)) for s in sources]
+    included_by = [s for s, p in source_preferences if p is True]
+    if included_by:
+        return included_by
+    return [s for s, p in source_preferences if p is None]
+
+
+def _source_preference(package: str | None, source: RepositoryConfig) -> bool | None:
+    import fnmatch
+
+    if package is None:
+        return None
+    key = normalize_name(package)
+    if any(fnmatch.fnmatch(key, pat) for pat in source.include_packages):
+        return True
+    if any(fnmatch.fnmatch(key, pat) for pat in source.exclude_packages):
+        return False
+    return None
+
+
+def get_file_hash(filename: str | Path, algorithm: str = "sha256") -> str:
+    """Calculate the hash of a file with the given algorithm"""
+    import hashlib
+
+    h = hashlib.new(algorithm)
+    with open(filename, "rb") as f:
+        for chunk in iter(lambda: f.read(8192), b""):
+            h.update(chunk)
+    return h.hexdigest()
+
+
+def convert_to_datetime(value: str) -> datetime:
+    if "T" in value:
+        return datetime.fromisoformat(value.replace("Z", "+00:00"))
+    return datetime.strptime(value, "%Y-%m-%d").replace(tzinfo=timezone.utc)
